#----- a procedure to show error 
proc error_exit {msg} {
  wm withdraw .
  tk_dialog .error Error $msg error 0 Exit
  exit 1
}


#----- get startup directory and name
set cmd_name [file tail $argv0]
set cmd_dir  [file dirname $argv0]
if {![file exists $argv0] || [file isdirectory $argv0]} {
  if {$tcl_platform(platform) == "windows"} {
    set sep ";"
  } else {
    set sep ":"
  }
  foreach i [split $env(PATH) $sep] {
    if {$sep == ";"} {
      set i [join [split $i \\] /]
    }
    if {[file exists $i/$cmd_name] && ![file isdirectory $i/$cmd_name]} {
      set cmd_dir $i
      break;
    }
  }
}
set curdir [pwd]
if ![catch {cd $cmd_dir}] {
  set cmd_dir [pwd]
  cd $curdir
}
if {$tcl_platform(platform) == "windows"} {
  set cmd_dir [file attributes $cmd_dir -shortname]
}


#----- set path to tcl scripts
set auto_path "$cmd_dir $cmd_dir/common $auto_path"
if {[info exists env(CG_LIB_DIR)]} {
  set auto_path "$env(CG_LIB_DIR) $auto_path"
}

if [catch {do_config -win} msg] {error_exit $msg}

#---- set global data structure
#WorkShop,the display region in real world
set  Xmin -1.0
set  Xmax  1.0
set  Ymin -1.0
set  Ymax  1.0 

array set ProgData {
  cgnsfile      ""
  obj           "mesh"
  solname       "" 
  display_model 1
  box_length    2.0
  background    {0.0 0.0 0.0}
  oglwidth      640
  oglheight     480
  xlast         320
  ylast         240
  fov           30
  scale         1.0
}  

array set Perspective {
  fov 30
  fd  1.0
  np  0.4
  fp  3.0
  ar  1.0
}  

array set Viewpoint {
  x 0
  y 0
  z 0
}  



#----- create main window
wm title . "TIF Tool V 1.4"
wm protocol . WM_DELETE_WINDOW do_quit

proc do_quit {} {
  exit 0
}

#----- create menu
menubar_create {File Display Help}

#----- file menu
set m [menubar_get File]
$m add command -label "New" 	    -command clear_data -state disabled
$m add command -label "Open TIF..." -command open_TIF
$m add separator
$m add command -label "Save CGNS..." -command save_cgns -state disabled
$m add command -label "Save solution..." -command save_solution -state disabled
$m add command -label "Save parameters..." -command save_parameter -state disabled
$m add command -label "Quit" -command do_quit

proc clear_data {} {
  global ProgData
  set  ProgData(obj) "mesh" 
  set  ProgData(solname) ""
  .toolbar.but.file.new configure -state disabled
  .toolbar.but.file.save configure -state disabled
  .toolbar.but.obj.mesh configure -state disabled
  .toolbar.but.obj.next configure -state disabled
  .toolbar.but.obj.previous configure -state disabled
  foreach i {0 3 4 5} {
    menubar_state File disabled $i
  }
  foreach i {1 2 3 4 6} {
    menubar_state Display disabled $i
  }
  C_reset
  reset_view	
}

proc open_TIF {{filename ""}} {
  clear_data
  set FileList {
  {{TIF Files} {.TIF .tif}}
  {{All Files} {*}}
  }
  if {$filename == ""} {
    set filename [FileOpen "Open TIF File" \
      "" . $FileList]
  }
  if {$filename == ""} return
  .toolbar.but.file.new configure -state normal
  .toolbar.but.file.save configure -state normal
  .toolbar.but.obj.mesh configure -state normal
  .toolbar.but.obj.next configure -state normal
  .toolbar.but.obj.previous configure -state normal
  foreach i {0 3 4 5} {
    menubar_state File normal $i
  }
  foreach i {1 2 3 4 6} {
    menubar_state Display normal $i
  } 
  C_read_TIF $filename
  reset_view
}

proc save_cgns {} {
  global ProgData 
  set filename [FileSave "Save File" "" . \
      {{{CGNS Files} {.cgns}} {{All Files} {*}}} cgns]
  if {$filename == ""} return
  set $ProgData(cgnsfile) $filename
  C_save_CGNS $filename
}

proc save_solution {} {
  global ProgData 
  set filename [FileSave "Save File" "" . \
      {{{ASC Files} {.txt .dat}} {{All Files} {*}}} txt]
  if {$filename == ""} return
  set $ProgData(cgnsfile) $filename
  C_save_solution $filename
}

proc save_parameter {} {
  global ProgData 
  set filename [FileSave "Save File" "" . \
      {{{ASC Files} {.txt .dat}} {{All Files} {*.*}}} txt]
  if {$filename == ""} return
  set $ProgData(cgnsfile) $filename
  C_save_parameter $filename
}



#----- display menu
set m [menubar_get Display]
$m add command -label "Reset Screen" -command reset_view
$m add checkbutton -label "Display model 1" -command   set_display_model \
  -variable ProgData(display_model)   -onvalue 1 -offvalue 1 -state disabled
$m add checkbutton -label "Display model 2" -command   set_display_model \
  -variable ProgData(display_model)   -onvalue 2 -offvalue 1 -state disabled
$m add checkbutton -label "Display model 3" -command   set_display_model \
  -variable ProgData(display_model)   -onvalue 3 -offvalue 1 -state disabled
$m add checkbutton -label "Display model 4" -command   set_display_model \
  -variable ProgData(display_model)   -onvalue 4 -offvalue 1 -state disabled
$m add separator
$m add command -label "Save Screen..." -command save_screen  -state disabled

proc set_display_model {} {
  global OGLwin
  set_view $OGLwin
}

proc save_screen {} {
  global ProgData 
  set filename [FileSave "Save File" "" . \
      {{{TIFF Files} {.TIFF .tif}} {{All Files} {*.*}}} TIFF ]
  if {$filename == ""} return
  set $ProgData(cgnsfile) $filename
  C_save_screen $filename
}

#----- help menu
set m [menubar_get Help]
$m add command -label "About..." -underline 0 -command do_about

#---------- toolbar
frame .toolbar
pack .toolbar -side top -pady 2 -fill x

set tbbut [frame .toolbar.but]
pack $tbbut -side left

set b [frame $tbbut.file]
pack $b -side left -padx 5

image create photo img_new -data {\
R0lGODlhEAAQAIMAAPwCBFRShPz6/PT2/Oz2/OTu/MzK/OTy/Nzu/NTq/NTm/Mzm/Mzi/MTi/Lze/Lza/C\
H5BAEAAAAALAAAAAAQABAAAARcEMhJKw04YyuDGCBBbFYQikVgBKV4HEUqrFVwEDCCaHVxIAVEQqGgXYK6\
hHJhnAR0iuFiwWh2hokptWEFBBRaRqPh6Aa044ejzLqIG4+x42E22O/3tlPD13P+fhEAIf5oQ3JlYXRlZC\
BieSBCTVBUb0dJRiBQcm8gdmVyc2lvbiAyLjUNCqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRz\
IHJlc2VydmVkLg0KaHR0cDovL3d3dy5kZXZlbGNvci5jb20AOw==}

image create photo img_open -data {\
R0lGODlhEAAQAIMAAPwCBASCBMyaBPzynPz6nJxmBPzunPz2nPz+nPzSBPzqnPzmnPzinPzenAAAAAAAAC\
H5BAEAAAAALAAAAAAQABAAAARTEMhJq724hp1n8MDXeaJgYtsnDANhvkJRCcZxEEiOJDIlKLWDbtebCBaG\
GmwZEzCQKxxCSgQ4Gb/BbciTCBpOoFbX9X6fChYhUZYU3vB4cXTxRwAAIf5oQ3JlYXRlZCBieSBCTVBUb0\
dJRiBQcm8gdmVyc2lvbiAyLjUNCqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRzIHJlc2VydmVk\
Lg0KaHR0cDovL3d3dy5kZXZlbGNvci5jb20AOw==}

image create photo img_save -data {\
R0lGODlhEAAQAIQAAPwCBFRShGRmzPT6/Oz2/OTy/Nzu/NTm/AQCBMTi/Lze/Mzm/KzW/NTq/LTa/AQCxO\
Ti5Nze3Nza3MzOzLy+xNTS1MTGxMzKzAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQ\
ABAAAAVkICCOZCkGaKqqpxAMRFEYRx0gQfvGRZ0oKZ2MdlgogC6doVc8MgJJADRQaxidU13t8HMwnlGoa4\
UShM2PtFp9fkAikomcQnm0IebKxGKp3/MTF3R2OVICboB7foVSZGQmkCR+IQAh/mhDcmVhdGVkIGJ5IEJN\
UFRvR0lGIFBybyB2ZXJzaW9uIDIuNQ0KqSBEZXZlbENvciAxOTk3LDE5OTguIEFsbCByaWdodHMgcmVzZX\
J2ZWQuDQpodHRwOi8vd3d3LmRldmVsY29yLmNvbQA7}

image create photo img_next -data {\
R0lGODlhEAAPAIQAAPwCBARCZKze7Kza5IzK3ITC1KTS5Hy2zGyuxGSmvFyWtFyavEyStEyOrESGrDx+pD\
RynCRqlBxejAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQ\
AA8AAAVOICCKQTCeKBAIgpmiwUAM7ksORUHbpHH8LV6AgCgidq/AIcFkImEJhWIxXdROAQWD0eA2rqNAw0\
Emg8OPdPochrgh7HBkHg9LJHVs3R4CACH+aENyZWF0ZWQgYnkgQk1QVG9HSUYgUHJvIHZlcnNpb24gMi41\
DQqpIERldmVsQ29yIDE5OTcsMTk5OC4gQWxsIHJpZ2h0cyByZXNlcnZlZC4NCmh0dHA6Ly93d3cuZGV2ZW\
xjb3IuY29tADs=}

image create photo img_previous -data {\
R0lGODlhEAAPAIQAAPwCBARCZKze7Kza5IzK3ITC1Hy2zKTS5GyuxJTG1GSmvFyavFyWtEyOrEyStESGrD\
x+pDRynCRqlBxejAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQ\
AA8AAAVNICCOYhCQKBkIwpmqAzG4LxAMRTHX9mD8BxrqhigiEsLRTYFQOA3J0mLBmDIUUVvDsd0ysraHWN\
wA2yBotNkWaUfWNokcbptM6LaoPQQAIf5oQ3JlYXRlZCBieSBCTVBUb0dJRiBQcm8gdmVyc2lvbiAyLjUN\
CqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRzIHJlc2VydmVkLg0KaHR0cDovL3d3dy5kZXZlbG\
Nvci5jb20AOw==}

image create photo img_mesh -data {\
R0lGODlhEAAQALMAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDAwP8AAAD/AP//AAAA//8A\
/wD//////yH5BAEAAAgALAAAAAAQABAAAAQqEMlJq5UgX6oxuF0VhuCHfKR3YlM3spzZwpxI\
z/Vq56nc+zcRKlXaGC0RADs=}

button $b.new -image img_new -takefocus 0 -command clear_data -state disabled
set_balloon $b.new "New File..."

button $b.open -image img_open -takefocus 0 -command open_TIF
set_balloon $b.open "Open TIF File..."

button $b.save -image img_save -takefocus 0 -command save_cgns -state disabled
set_balloon $b.save "Save CGNS..."

pack $b.new $b.open $b.save -side left 


set b [frame $tbbut.obj]
pack $b -side left -padx 5
button $b.mesh -image img_mesh  -command draw_mesh -state disabled
set_balloon $b.mesh "Show mesh"
button $b.next -image img_next  -command draw_next -state disabled
set_balloon $b.next "Display next data"
button $b.previous -image img_previous  -command draw_previous -state disabled
set_balloon $b.previous "Display previous data"
pack $b.mesh $b.next $b.previous -side left 

proc draw_mesh {} {
  global ProgData
  set  ProgData(obj) "mesh" 
  reset_view	
}

proc draw_next {} {
  global OGLwin ProgData
  set ProgData(obj) "data"
  C_draw_data $ProgData(display_model) next
  set_view $OGLwin
}

proc draw_previous {} {
  global OGLwin ProgData
  set ProgData(obj) "data"
  C_draw_data $ProgData(display_model) previous
  set_view $OGLwin
}

set b [frame $tbbut.text]
pack  $b -side left -padx 5
entry $b.ent -width 16 -textvariable ProgData(solname) -state disabled -highlightthickness 1
pack  $b.ent -side left -fill x -expand 1 -padx 2

proc tcl_set_solname {name} {
  global ProgData
  set ProgData(solname) $name
}

#----- OpenGL window
frame .plot -relief sunken -bd 2
pack .plot -side left -fill both -expand 1 -padx 3

set OGLwin .plot.gl
if {[catch {eval OGLwin $OGLwin -width $ProgData(oglwidth) \
  -height $ProgData(oglheight) } msg]} {
  error_exit $msg
}
  
proc set_ogl_region {} {
  global ProgData Perspective Viewpoint
  global Xmax Xmin Ymax Ymin
  set Xlength [expr $Xmax-$Xmin]
  set Ylength [expr $Ymax-$Ymin]
  if { $Xlength>$Ylength } {
	set ProgData(box_length) [expr 1.1*$Xlength]
  } else {
	set ProgData(box_length) [expr 1.1*$Ylength]
  }	
  set Perspective(fov) $ProgData(fov)
  set Perspective(fd) [expr 0.5*$ProgData(box_length)/tan(0.00872665*$Perspective(fov))]
  set Perspective(np) [expr 0.1*$ProgData(box_length)]
  set Perspective(fp) [expr 1000*$ProgData(box_length)]
  set Perspective(ar) [expr $Xlength/$Ylength]
  set Viewpoint(x) 0
  set Viewpoint(y) 0
  set Viewpoint(z) $Perspective(fd)
}	


proc tcl_set_post_oglregion {xmin xmax ymin ymax} {
  global Xmax Xmin Ymax Ymin
  set Xmin $xmin
  set Xmax $xmax
  set Ymin $ymin
  set Ymax $ymax
  set_ogl_region
}

proc set_post_display {} {
  global  ProgData Perspective Viewpoint ViewMatrix
  global .plot OGLwin
  global font1 font2 font3
  set_ogl_region

  pack $OGLwin -side left -fill both -expand 1
  set ViewMatrix [$OGLwin get modelviewmatrix]
  eval $OGLwin eval \
    -clearcolor $ProgData(background) \
    -disable cullface \
    -matrixmode projection \
    -loadidentity \
    -perspective $Perspective(fov) $Perspective(ar) \
      $Perspective(np) $Perspective(fp) \
    -matrixmode modelview \
    -enable depthtest -enable normalize -shademodel smooth \
    -clear colorbuffer depthbuffer \
    -loadidentity  \
    -lookat $Viewpoint(x) $Viewpoint(y) $Viewpoint(z) \
      $Viewpoint(x) $Viewpoint(y) 0 0 1 0 \
    -multmatrix $ViewMatrix
  draw_screen
  $OGLwin main

# new bind  
  bind $OGLwin <Configure> {Pers_ComputeViewport %W}
  bind $OGLwin <Any-ButtonPress> {Locate %x %y}
  bind $OGLwin <B1-Motion> {Pers_Rotate %W %x %y}
  bind $OGLwin <B2-Motion> {Pers_Zoom %W %x %y}
  catch {bind . <MouseWheel> {Pers_WheelZoom %D}}
  bind $OGLwin <B3-Motion> {Pers_Pan %W %x %y}
  bind . <KeyPress-r> {reset_view }
  bind .plot <Enter>  {%W configure -cursor exchange}
}


proc Pers_ComputeViewport {w} {
  global ProgData Perspective
  set ProgData(oglwidth) [winfo width $w]
  set ProgData(oglheight) [winfo height $w]
  set ProgData(scale) [expr 1.0 / double($ProgData(oglheight))]
  set Perspective(ar) [expr double($ProgData(oglwidth)) / \
      double($ProgData(oglheight))]
    
  eval $w eval \
    -matrixmode projection \
    -loadidentity \
    -perspective $Perspective(fov) $Perspective(ar) \
      $Perspective(np) $Perspective(fp) \
    -matrixmode modelview
  set_view $w
}

proc Locate  {x y} {
  global ProgData
  set ProgData(xlast) $x 
  set ProgData(ylast) $y
}  

proc Pers_Rotate {w x y} {
  global ProgData ViewMatrix Viewpoint Perspective
  set scl [expr $Viewpoint(z) / $Perspective(fd)]
  if {[expr $scl > 1.0]} {
    set dx [expr $x - $ProgData(xlast)]
    set dy [expr $y - $ProgData(ylast)]
  } else {
    if {[expr $scl < 0.001]} {set scl 0.001}
    set dx [expr $scl * ($x - $ProgData(xlast))]
    set dy [expr $scl * ($y - $ProgData(ylast))]
  }
  set ProgData(xlast) $x
  set ProgData(ylast) $y
  eval $w eval \
    -loadidentity \
    -rotate $dy 1 0 0 \
    -rotate $dx 0 1 0 \
    -multmatrix $ViewMatrix
  set ViewMatrix [$w get modelviewmatrix]
  set_view $w
}

proc Pers_Zoom {w x y} {
  global ProgData Viewpoint
  set scl [expr 0.01 * $Viewpoint(z)]
  set Viewpoint(z) [expr $scl * ($ProgData(ylast) - $y) + $Viewpoint(z)]
  set ProgData(xlast) $x
  set ProgData(ylast) $y
  set_view $w
}

proc Pers_WheelZoom {d} {
  global ProgData Viewpoint ViewMatrix OGLwin
  set scl [expr 0.001 * $Viewpoint(z)]
  set Viewpoint(z) [expr $scl * $d + $Viewpoint(z)]
  set_view $OGLwin
}

proc Pers_Pan {w x y} {
  global Viewpoint Perspective ProgData
  
  set scl [expr 0.001*$ProgData(box_length)]
  
  set Viewpoint(x) [expr $scl * ($ProgData(xlast) - $x) + $Viewpoint(x)]
  set Viewpoint(y) [expr $scl * ($y - $ProgData(ylast)) + $Viewpoint(y)]
  set ProgData(xlast) $x
  set ProgData(ylast) $y
  set_view $w
}

proc reset_view {} {
  global ProgData Viewpoint ViewMatrix Perspective OGLwin
  set scl 1.0
  set_ogl_region 
  $OGLwin eval \
    -matrixmode modelview \
    -loadidentity \
    -scale $scl $scl $scl 
  set ViewMatrix [$OGLwin get modelviewmatrix]
  Pers_ComputeViewport $OGLwin
}

proc set_view {w} {
  global ProgData Viewpoint ViewMatrix
  eval $w eval \
      -loadidentity \
      -lookat $Viewpoint(x) $Viewpoint(y) $Viewpoint(z) \
              $Viewpoint(x) $Viewpoint(y) -2 0 1 0 \
      -multmatrix $ViewMatrix
 
  draw_screen
  $w main
}

proc draw_screen {} {
  global  ProgData
  if { $ProgData(obj) == "mesh" } {
  	C_draw_mesh
  } else {
  	C_draw_data $ProgData(display_model)
  }	
}

set_post_display

image create photo img_about -data {\
R0lGODlh0gA+APcAAL//0Lb/yKzsvJfmuYKzwg0ODITtm8D/0rT/xqX+uITF\
tIrZt6P+tarmy6H+tKf/u4/noJLC1K//wofwnaHssrz/zoq8ypzsrZ3kq5XZ\
xoO5w6n/vJrzrI32orj/yp/+snu8vZz/sZTLyZb1qYrDy37BuKv/vqjd1cT/\
1rHtxqYQDqz/v6PdywoKt5r/r6j9u5Lmo7H/xJPeqK7/wKT+tqz/wJf/rLHy\
wa/zwJj/rbr/zJ7zsJMKCaLmspP8qK7/wnixuZX+qhUUxwkJrJHvpJX4qpH6\
p33DwLHr0YHBwoW9upLupIzbqrb1xsL/06bzuK3xvqL4tLD1wX6yv6b5uI/4\
pbL4xJb/q3WxtrXyz7D4wqr+vJ/1sbT5xpTxp675wLXxxpL1ppP6qJDsoqD/\
sqL/tK31vpn6rBQUuKX5t7D/xKD5sn+8wKTxtpzO1Z35sH62v6r6u5TdvJrI\
2pLTsYTMrXi2uqb+uZjF26X2tpDponazt5/wsZjuqoPLvpbE143EwYv0oZT9\
qqPqvK3l0qj4u5Tvppv/sKf2uaPiu5/mrpT9qYe8yLj1yX2+tIi0yJ/jxqL0\
tKz4vorTwbP/xoXCx5DdtY/hs4q7xLj5yaX8uKjwuaXxw5PrpJTopZ/YwZnb\
saL7tJv3rbP7xY/Kt5LTu7L1w5XI1I/joKT8t4nzn6r7vZn/rpLmr5G/0qP8\
tnmzuq78wZf8rJrQw4HqmIzPwn/olqr1vI67zpr7rqj8u6vxvKH8tI/Cz5/8\
ssERDpz7r5b7qqLwtIe2x6nnu6/uv4YXSbz30Kf8up7S0Zj4q5T0p3uxu5HH\
yZHxpJv9r7T0xKfpt4a2xp3+sXi5ubD8wrDwwLTyxJ/7saD3s7v7zKv9vZr+\
rRUVpo/qoH4YS3u4vAkIop3/spL7p5//tI73pJD5ppD5paH/tqT/t6HP2o/v\
orf8yLPww5T3qJL4p5f5q6X/uZ/zvYDKjXe3uLP9xL79z8D805nlqJnoqZn9\
rVsZOq38v6LcwobHyI6/ziH5BAAAAAAALAAAAADSAD4AAAj/AAsIHEiwoMGD\
CBMqXMiwocOHECNKnEixADly6NCl2xgv3oOPDzaI3GCi5IoaM1L+kCBBjZoY\
MGNQmokAQYCbNz3o1Kmjp44KQIECAHCg6AEnTlAkRcG0qdOnUKNKnfr0F9Sl\
VJkmRerE6FAAQSv43KkT582aMynFjOGS5Y+UM2qsKGli5AaQDzqmU8Eg40WB\
FzNq5OgRpF26J+GubPlyLU2bZgOQ9eDzZ9gKX78aPcq1s2euWUNHTWr1s2mu\
m70OvQx0LNnIAdCqXdtWwtuUcunaxas3lArBfwuIC5xxY7qOeA+bRAmXJWPH\
lGpCxjmZsk/WmlMXPc0dqejQTn5x/+eMVHvRzKsv95wcWfpM2mqcw407t+Ru\
kHo3qgiF7qI4gcMRN9hxhX2knAmJKcaSS9BFNx11k10XFHpDmUded6B9R1V4\
nm1WnnkUgsXaejvBJttsMNV2G2711TUSb/EY58tvF5Hzn3ABCmYcgcmNRBeC\
8y3GYIM1wRZhT5dRaKFRqHnInVJTbfXZL0uCGGJ6QblWllknrsVWfLbN16KL\
IsG4IzoqYOPfjeIEiFFxxiFnmI+IMZeSc0PG9NiDOfGEZFj2IHGCG4S6cQIS\
9hB1FBLHVHnaPYKqI+mkh97TGQpIWMrheUsCAOkJ6hTqRjKHahOWDkho8xpO\
7qGYIpgSzP9XQw0t6PYifjEah86MNbYpUAht1qgjYQWGdGCCKi3YmJ6PGemn\
ZRUQcgoef1RLbbVznECIoHhkmpmFXB1zwhxz4GHuuedmm8U9kM7R6AFUnpfd\
V+KWW+201eZ7yglZaIPqKVl4IBmXCKTlpYqyztVCDXbdhWuu6QjGgwM2ihPC\
r226OSyBxTaM2ArzzYDnsjClVSSEz2bhhr4sINEAEiy4EUEE+f5xgj04J4oe\
k04gUS4e6miLxNCEDJovut3i/MuVXxFiL6nbvtxAzDNXfW8K7LDDpcHwySfm\
SbQ2DCPEgjmgQsUWC3QIsBpvzOOcdC4XpLIky0STWTv1lMUpf5z/EgELTQQO\
DTSBN/BH1RG4oc3iOO9s1AnmqpPqMY1UbnkjYCRzeM2E2KPNLxXoTCHkf7iB\
RCaNCA6NKaunkMw/M/cycwNdjKK1ewcjDFdudC1860f5bSRYfzyU0WYIh6jN\
dttwEoaXsSL9iOydynpp94M77e13BA1Ac8MN1oR/QzvXpHCKK+hH0EQjmWgj\
+reEmOtG5UKsD8b9+N9/Avr8s9CIEL8wFYWiVS03qM574BOfNW7Agn840IEs\
MIUVRjGK6LjqS17bXX1IEjaH4epMgiGD2TKGPIG4YG0ZE5bb5AS36NVpbtVb\
i3QglIlT+K0XLABfMXa4w/CFTwCuwIUQ/1NwDfbZQyiZycK1spA6IYChHVCM\
ohT7IUQhCkEI7HNfWIaShXyBoQkI9KEPeViMfljgjBaYBQ6koIVR1OM9r3LO\
iujzo5HU6nnBixhwLsKDD1jsEC4w4dqWp8Lmva2FJHnhfEa2rLvlJADJiF0y\
rCGASgqghwscXxkfwcl+WAMa7UuSPVYWgRM04RrtEEIUr8FKMFzjie1IwSxw\
cUURFOMaTVjcZUZ5OBZA4xrWKEb4oCBGHFgDCsUQwCyGMQxpNEMAu9CCFuoR\
E5fAKkgtGJNdWsDNbnrzm94cggoshrxAFoAVJ1we8wbEsR4dCEF2oh5jlnWy\
ADQgdr2oZDQEEP+NYkDhBjgwhUB12AxOzkIAuGQHawhRrQikAIrFEMINrjG4\
wAkOlakUAif/IYB2gNJfYWHozFIQvh4CVAooRSkOcACFSmKCAAQARDQ2YYYv\
VCOOLAmZXFrwA/s0rAUO8wjE9OgXcpChTdOYRjlZIRB0HgKFKRQQOztSLOi5\
0CTTW0xLShadm4zihv/oBz+jsU8oBNQKaO2CKYKZCE4CQgA3ACW0gEJKN7SD\
h0LAZRO6kIm+ZiJwV9zkIzBBjGJAowv+sowOZBYBEQTzkguUAloni1aU7mIT\
/YCpBnqwiV3Y9CVyTNisakWmF7UAeHkcXq9I+FQXMLUANnAqVN30JkP/HtJA\
HlOkgrYqkyKxIAK9+EcvyEpWaOJggrYbRRfUegMBFJSw1jAFYqHViMO54hPJ\
FKYQTJEJdmjDu969IhibCwhOEgOufP1JT6obgX9gt5ILlCAF50tBaeJgF9EA\
BEx7EI0nSCIWoU1Y7xh2nwecNo9E7c9q/whIVuRAIDlghVNDQMhCsrOd7ryq\
3BbZGAt+NbgW+ARxo7GL47oRJ1lTKxQ+wczC4qAL7KgMEiKAvk9Ushg3EAJi\
2aETdlzRAymGhjXa+ohEwNUKPF7PjCOACxtXco1IrkerKFGPamjBDJtIxH4p\
sAlJ8COnAqZLB8uElxYgWLULpnCDWWEDgVwh/8IuSGeFpTpVFuL2nStA1oJK\
hgBI9KIXZ+QvfzeBgzZKOTLssIIUisGIYSSiGMdNck/u+Y8m39gUQuCxToTQ\
ggpoOsU4KEYzHC2A6CZZJ5S29CVNLJ1Wq6XKX4DCJgCxWS7fgh86zXMdN7Cw\
kODRzCDc45rICUjX5qDNsM0BnJ9K4ahaeEe3vXPcEJQglvBZBMFlhEz5S2JJ\
TPN6kEHAKLSAAxFIw8YmJksDgtjk834y0z4WAmUmkzUr3IAYxNhnQEeh7uAO\
Q8QCgIIUTjzlmFRZEpsYBH8p8IRba1CbIgnb8/Iy1ATXaNhqbrCyr+BmG0RY\
ts12dm0vTNXnNSyRH/8T2VYpwYkIkMACjChFDwZ9C5s6xj31GIUUiGFkQk9Q\
MkBeNy6GMYvz3gDTV8wa0BGt85Ya9+c9bsA/LDAMEZR14LHhGm2q8YVbbKIN\
FGjDJgqxipPoetd2xOOZhY02Yq/52BwvwBVs4HEJn3CQGaPt8Fqwo5KbnNfT\
BpnKYyABFgC60aCY+UwlcVPrOYgS4zaDrEvcRgTsROpDJyx6hbB02OBkFIqG\
guglOwqzSJ3qmChr5QNwwTjGQhKIeEIb2oAIsiPoR6Ut02kfFuyiDtvtrnUw\
3eMehLkr2+7MVufFBdPNvtv5I0BVjuCtHQMRvJwRBJg5Zz0L4LrZLQb10ML/\
F8ywi12YYfUBYIcppj6MRxz0BlaQt8AE5vkA5FwLUjDD+SkIGfWz3/0BV3lo\
AR9tEQv8UAixV3txsApbUAO2UmC7R1XQNjwK9nsZF3zKZgNXEAQCEQTF53Fw\
dnchFywXVwYtkBHcFCd+B31WlUiCNwMxoAXXpwGYoAgztwk1133W5H1cJwn6\
9wXop1zNYAHS8AjnZg0+NjD1Jx1VJk3i50aQAXpDKA1U+AlQIIBwFBPOESv8\
sApxUAiFEAe6sA1b4FMFBn1UNVQUeHFtd4HGBnceKBCL8IF1h04iqHwX1wJl\
wHzOVyC7Z1VAIjIzMAjXF1MY0AMM52VaRTeNoQbV/xALXyAJX2BTb2QT9WAF\
ZtR+0kB0OTZ/JuJq9VBl1VAN1NRV4dcPjCANBECFswAFbvQeagErsZISK7AN\
26ALuEiGuedBZbZ2bNeGyFNsEnZsGhgEiyCHgkCHHxdnyZd3AUIGJ7h3fKeC\
HvGHdwYkEhALkEACjKABBEAKh8hwhYBrybKF8yQBoxgLsTCKNzUTlygFJDAM\
MPUIFtAMQmA7ntclMDGK0BF+ZkACqriKjEACkOBGb1RNGVQftrgFDCk2E9cR\
06gra7hgxxOMcTaMwxcEgnCMBeADc6iMdneHzngRLUAGI4cOEckx1nhneZYS\
GQBzGjAFpGCDwnALXhYyM//wA4toG9mojtWwclQmTcTwUjA1DBYgBG7QAKNQ\
JK6WhSliPfUwfsTgjTAlDWckApBAilpIeLM4A7rWkA6pdhDZexX4e8CHgRlp\
jD4gED7gkR44d3XIjIOkTtBoYSiZkvFgZlUVPSDDDyLACNg3BXRgg7QXB7gW\
T81hjltoPdUgCbuQCERJAEfJCL0gAkrJlK1iPV6Cjo7ZA5EpDYxgASSAlT8p\
AVwJF2d3chNHcfkxjRTYHy1AkRRmkWhZjGq5lgVgBB6ZjFcAl8t4d3iXMSW5\
fNJoHHrJQj6yAvzQDJXgjYKJAWGHCKtAbWATMjs5T0OCjuO3CT1ACjBFAEIw\
df//QALNAAld0JTVJIsMko2S4HXdOQWaFZqiWZ5fkJAb9DtjM1R8N5HkUJLO\
mHHCKHwauIEb6QPjIBBGMA67WXy+KWF22IxtMgS0RWfTOI1paBgmsAXbUAnd\
OAXPGXa21yJ59oLypJjUx3XkFw09AApKAJ5UeEbCRQIsYAWhKBO5o5iQ6HUU\
sKJKAJ8aoAGASQLkmQGSgJq6hp8PU3HpEI2+Z1Qt8J+06aDHNgS9qZGLYKDm\
IBDmoKBuyaB093EPOpfi8KQkaJfNp4JzsgWrwKExyQx0AJ20x4DSU52LZI46\
aY6v53Uq2gN0gAZVOQyhOZ4yKgXUpIXmqBIpwQ/tCXaK/6AIdBCTBPCjgDmQ\
JECk/DCiZpif0KZHTMqGYzoNFRmlEjYEpFqltzkORqClPLCqrNqqrtqqQ9AC\
pJp3bFhb6UADPKACurqrvLqrxmAM3xCsKrAPFBAJVKALZVgXH/OCW3gnMdCr\
0MqrdNCiqwikZ0QCvZCV1WBNGbQ7XSgJTyAMFKAIGIAB0+qjQBqkBLkKuvg7\
abipe1eWayKhbieMpDoEw7eBp2oEVSAQVWAECdqWc0h3HscN3HAI03AIZDAN\
ZBACLhACZEqrbHiXLcAAoXCxofAKCeARJDEXzQkHU8AMC5APxVoIyCp9LZmY\
d3InhQoTVaaosdcGwoAGPeCo1P8qDdYqmiJgBqXZlV45og7YhYUQruNKruba\
owQAB5JaCZUgAohAhlvAiwiWYChokhTZArP5VIB0rxk4oB54pQYKsOfgr+aQ\
oFz6kb35pWAql0Mwgs7GTeRggsODphuwBVTwsczgptAZCWmgC7mFVTj5NSM6\
okIbs2hgg+XKolWZrqLJCesoiHEhF4NrAtuwCggIdhdgtBjAoh6qtEBaCcYQ\
rdAKTt5EkREqq7J6r6q7uqo7DqhqDuUgEOVwDmXruh6Jtr65tic0BGJKq8NJ\
kns4t8ixAS9wtxoAB3lLByRrrMhQJnZRCJwQvdI7vdJ7C5KbZ7NCuV4Ye2hA\
AUVbrtP/qlnpSgKDwA/kiKm4twUvoAtUEHt88L34cLSdCwdswAj+IA91ATx3\
kAo08AqhwAt9gWb9aZJQyrvF1mD3mq9fe6Xh8Lrn0K8FUA5VcA4Aa7uLgLtq\
u4yHYMDBKQ6xSluxSYEccQcPULyVwAbICwt0gA/FmgYv4GvQAwn+4A9CWsM2\
LKQZsA0ZinslsQ1xgAhoQLTfa64gG5+MwAmrcKleaYYjwZDsmwZ50Abvmw+K\
EL/4QAfIOwWemwTCQAWacAf58ZplCY3OSE4hwLtxhoEZeK8LbKBDILZV0AEC\
0QESTMEBe7tvCZcg6KAcPJdDQA4V9ru2ehwPgAxU4A8onLws/8y3JNwjg5AB\
GTAJJFAJQkrJOJwBkKAJuvACyfojGroKaIAIiCCz3tuom9uj6AoIYajDaGcX\
L/AChpwGkRAJ75u5+IAPMqAEeavFbOAPeZAGmpAAwVarNtICoBqqT4XGUkqM\
XnuvbTkO4WAED1wOsVsAdFwFVVC2d3zBHkiHe+wCQ5DGyYy16tQmscmGe8gR\
CZAGtZDIQEAK+HABfKsJwAMSdwDLljsITMsIHAoIbZAHAE0Fx8rJJ0cSLaAL\
cdC+kSCu41quCuChU/CjcoAIcUCGBT1xL6AJaSDLtHwBF5APGIAPCsAMsMAM\
9CsHURAKCUADEbOHxJwx9Jq1WhvOy/9MsPpqjAz8xkNgDthcDnJcAIFwzbRb\
wQL7kV4KgjkwBHZHqlrrtp+KzoMRD+tcCu6sAPEcCVGQCkJVGPdsyABdCz8q\
0cIgDG0QCb+MDK9c0LxmAusr0Hmw0KWMAQ/NDFrszxRt0Uhqz3eQABodBbPM\
Bx59ywoABCQNDkmwAyn9CgFcq85IzsHYWuD8hsRoqjj9zAAbDtPcAT+t2dQ8\
wdrMpQt61B6n1KwQznLZux4scuhQBhuRAKEgBygMC0DgCPHMBVm9sSV3B5pA\
BVAcCaVwvHAAB6CgCBTAB8Jg1lSADC0YPUD1ym1NBW8truSqy3QNBwOQB8ca\
tbyYnwnQ3an/EAp+vQOAnQ/44AhAAASwAAetYNuK7dKmG6EAmsY1rcBW2pZh\
awTmEA5VQM0dEAgCoQqcXceffbbJKNr36qDinHzAErHBYpKD0b8DwAbgcN6w\
EM+Izb8cI9W6DcVtIAfBHbIyoAgkywdmnQZoHZbR9wAk/AJo4NZgRwGgELJ0\
XQrCgN2vvJqsGSMdQQMJkAr+GwVcsAMeLQPMQOF0wAdcsAYMQDEEXMYU1raQ\
HXwO1rVeW98Giqr4fQ7hwN/9LRCBENR0LOAVTODdnLb4qmy6C5yzWcYXUQas\
zQCvkA0SLttAIAMXgNivwNI7kgAbLsuwLeMyEM/yPM8v0Mgmx014/3EHyIAG\
u/3W3jvSdK0ALYwMhp6fSkoD3x0F2SDe+TDYQLAHCtAHO7AGvEAxbReqZ1xs\
Uo7mNn3TBXrlAGsOD7zlmh0IqvDfYH7Nnj3mRV3merzHCH7aay6xq22xa+AH\
4EDnyrsD2RAKeh4nfK7RefDnJR3o+UDiefDFdyAHk7DdBmZg1ngHaODVtEwH\
Mq4EF1DjLvwR3J7jFQcnDNC/4M0FfEAHe3DejtAHffAGb/ABRpV3Mr3BazaM\
zFzllQ3rWY7N4aDZ/X3rBaAKqpDrErzrZlvUBS7aaB7swDmXhEQG6OAAvBAF\
lpDsQIAFCpAPzB4KDBAxO8LjGw7byFvn8f/MB1EAzCQ8w/JwBzoPEgeWl2b2\
EeIOy1QQCR6et0pw7XxL6Q+A8ytNA8EjxukA56GwBjtgCXYAC1jgCJ7QBxzQ\
7xUzm4+dzKtO5ZS9kWDrurH+wPu98A3v8BMQ8RKPzUNd8b2ex3qc8SF52mJq\
tWXgAGtwASAg21hAD/nQB1yQsQmmzrod4XBQ7RYeBdp+B7XgD3KgCV+M26e1\
I9yEHGiQ0dMe3LBw9Bdg29o++XKQCqj/7FQbvFFPA6GQDSMPC3vgCDDA9W8A\
DKAKoKoe2VPOzL150xds32if8PvdAQv/5RAvEG8P9wEu9wN+totw8WlLsHhv\
h3rPNmRgkg6ADVz/4Ad2cO91PvpKnvjqrAkDkASNX+f5MPppsPOaIAczXPPB\
vLFoUHF5iQYfwehpAPOwABAl8l3YEUXTnVRyKvmLEirVKxrp0qErU4YcOXQZ\
M6Zj8CqUJRB29jiC0YmDKGDTQhxiecjFSxesWA3JUdPGTRtXrgThKWjRIh8+\
xg01YsTcuSpVypXrEC5QIFWqJhQoMCGqqqcdOixNes5cUSNDg/r4yZOnTpw2\
auaQKRNmy0MhxIkj5wDbG0t2YGHBUucChzW8HFCsiE5ivASaBrABBwuWjIFc\
oiRAnEBhkgF5oqSijOZVPImhEaNB8wANMjmMYdXBR7BhAstJMEsO9Wqi/8aK\
FytW5BhqARuRdUp6QRmXJcy2rHLQTKvTbJCfY4kWPZp0qVanV6dWtRo1kNat\
5bp+LSo26E9BZnVeSbs2+du4c8l88PWGg6M9fKnl48AllIPCNpKIhlQWa2wP\
yC6QLJXKBkiCDT9oewiNdFpAo4UWRLvjDtLQqIUNOGBhIp98dsjGIRochNDE\
UHhhQCJ0LtINo93sqiWkPZgYTplcjnspubWGwGk9s4aILqjpjEJKKaaw0267\
CaK86jutuKrCK/KGGmcs9NRDCyf32npLrrk+eEOUBfbIDwsm+uBijcEI4y0d\
Gl4xEBYgZPDEzSgiqpOLB9mwhIsTE6AwtHQu7P+QNA6FYAwcfPAp8cRXXgEU\
BEEJDeXF3SzSrVMH1jgCBGqoQWXHXJBra62bhCSyp0WGkG46c6pjErxAwokq\
Sih59Y7K8MS7krywtjxvkfS8ZA/Mmt5zIS65psFGlD7w48uRPjh4Q7AAyzCM\
wEtUQ3DPHdZ4saMoHsT0gmz6PFSiFxFNYAPShBACDeH25CAbXiqN4gg2QABB\
QXMBzIgMMmQEdYBR6REOgk684JEbIG3CaQjnzPIpViTHAcvWpZrs4CldeTWA\
KgN8/RW88MYDS0sukX1uvfaaFRPaaYARhQMm1MRijwVOgrNT3jqaRFw9+9jh\
Dds6WsOPR4/g40Q0IkL/9Op0EqjlwRK66UbfwBjg5WkQwAGnhP7+8/QihBHe\
jRc/RnUEAlT0WEJiWShWzuKbdBriuY2DGkLLj5e8DleoSo7y5AJSVhkrYFse\
lliYj032rC9vWostVp49RJxn3lCmjzrWpEcGUd5w4AOiy2AgG4DByS9pbQEs\
g5dsFijbbAX4WGOeFzUSMDQ5ZANBBhg84SFb1R3AXXdw7ADniD4CAxBhGdsm\
A50FjigBBCYggMDuZeDRZ/PmnPs7iMCRHPxjkEUeGSpVwuF1AsYNcFzKlcGz\
ctiXmcdyM8uc5prluWfkQhQj6EQJ8rMHarRCFNhgHdF4MYlRiYRNnjBE7SgS\
/yqQ2EF64ADBJVrEgNbBSw5HANgCPOEJGPBAFBN0QA3XEEIRnu0S2PCFA8ih\
vR/Op3ujYoIexDcGQ7gjGOdLS04yFgQjHalj4ShcUm6FuKvYb3H509/9phS5\
kI2HcgIki8yUtSwDuoUbCRyBISCggPzQgxqT4IAvPvAB7A2gFiz0AxOYUIdT\
dfANH7hIqFqBoz1IDwR+GEA2aribjAzAD5U4QhIW0AcYGoIH2rJjXdZwSHqo\
SY6LvMQgCck2cUiShUdgQieM6I0xLKMIskjfE9knRaEQJRxD2GU4fPlLYAYz\
HDzYIi2oQgv97a87kGNZGF0WQGMNkIBoVEvnXMCNXP8oYwRL0EPp9iDHEvhh\
AeOcRNxYeAlMokKdMOhgShD2AWxwICT0UIACHEENEPBxEguQwwIm4Y8jANQP\
reiDKz3RCRmmZBrkgKc86UFPe+KTj+Mk5yqPMFAijAEC3vAGEcIQDFkEwYk7\
ARwuCVeUIZxjSUyS31OexINk6u+YybzfMp8CRmFdySvQjGYZL4e5LzWLG/o4\
wwi8sAS61cER9BCYRVl4SS8YQg+oEF/ElJGSuUhLFCVQgDpPBYPu5dOpLJzE\
JTjQB0N0Qq1eSOg0sqqzEpTAqzBAxQK+N1ayXmIZRFgCR72xjjAoMQiv0phJ\
T2qUcKz0cFq56VWkYgCYJtP/mAWgBTK76MVf4TSnWOLpWMpoRmWphS05kAU8\
jDoGb4gPFUyg6AIu0QoOjKCNrjSiHgwxAh65VRw5E8UlDEHXdXbCEK24xDgt\
4VrYjoADXiioWg3B1mk8Q7e8vQRSvQqxtBaXoq9dRhjCYIgloPavzlDiT2Fl\
2CTV6hzhsM5i50c/qSyOB5at7DEte9nHMbOZm+VseSp3LNACtZqkLcJexcvR\
dRDBC8uQrReiuoTwjmEMCh7BGXLxjLmArqhppRsEYGDb5zpYuRwgsYPB2wkk\
EocHueBGCFwcglxs2IhUHd8SQrwMHOPYwURYB2rHsA5nhEEMgvjpxoAyq/Qe\
BSns/3Xv/Bx7P1pE1gC0sAVVbFHZmNbUsVnZL39r1VnP/iTAg71JEGQRjAI7\
I6McHcMSiEAEQxjizRCGsIJleQZ9PIMbLAnBM86gjKOOQQ96GEMnbBznEJsY\
vHQ2BIPPwINpeC4ufjZqeNnc5jdnWtMZlfA6gPwOMZDlvEdGsseoo2QrhqPJ\
Ln2SyXhgiynXtwBXxjJNMZtZlgXLiir9MpgBTOTniNTMwRCDO5ahZgm32c2b\
XrAs4QEPWZzvJcfJhWmjimgHKxrRcV72c0dQhDMAgwdiegk3zmDtTadb0572\
dJCFPA5kGdmzWjL1qVVqxZDlurGtXlyUY22LKs+a1vdV5uYyIRe5YHmZWMUi\
42eBrTFZCCIY7vCuMyx+8Yvj2B1FKAJIz8c5NRLVtLIl+Qh07GBneCHlKcdx\
EeBxBlnkgAeHSI5y9BEMeBgbx97lOcZ97m5Qh1rMns1lkoyC6vaumtW7ul8y\
X11ZgFt54ATX8pa57L//6bTX/v3vr5NFZEGIodgU53nZ3aFEkNJyc2sfKs6f\
/fJnK6MIJSc5g10ObX3YgBs8WPtabo5mjgO+CGQve9nfAeotkbpjhz36vfGt\
dPgy3WT6s8V8oR5wqmRe85vnfOc9/3nQh170oyd96U1/etSnXvUBAQA7
}

proc do_about {} {
  global ProgData
  dialog .about -1 -1 "About TIF Tool" \
{             TIF Tool 1.4
 A post processor for Medici TIF file.
 Medici is Registered by Synopsys.
 Date   : 12/27/2005 
 Author : Gong Ding
 Email  : gdiso@ustc.edu} img_about 0 Close
}


if {$argc} {
  set file [lindex $argv [expr $argc - 1]]
  if {[string index $file 0] != "-" && [file exists $file]} {
    open_TIF $file
  }
}

