/*
 * tkAppInit.c --
 *
 *	Provides a default version of the Tcl_AppInit procedure for
 *	use in wish and similar Tk-based applications.
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <tk.h>
#include <tiffio.h>
#include "locale.h"
#include <GL/gl.h>
#include <GL/glu.h>

#ifdef LINUX	
#include <GL/glx.h>
#endif

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#endif 	

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <stdio.h>
#include "tifdata.h"
#include "tocgns.h"
#include "oglplot.h"

extern FILE* yyin;
extern int yyparse();
extern int yylex();
/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */
extern "C"
{
    int Tkogl_Init(Tcl_Interp *interp);
}

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None: Tk_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

int main(int argc, char **argv)
{
    printf("TIF dump tool 1.4, write by gdiso on July 28, 2007\n\n");	
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;
}


int C_read_TIF(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{
	OGL *ogl = (OGL*)pointer;
	if(argc!=2) return TCL_ERROR;
	yyin=fopen(argv[1],"r");
	if(yyparse()) return TCL_ERROR;
	fclose(yyin);
	ogl->set_post_bbox();
	Tcl_ResetResult(interp);
	return TCL_OK;
}


int C_save_solution(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{
	if(argc!=2) return TCL_ERROR;
	if(solhead.sol_num>0)
	{
		FILE *fp = fopen(argv[1],"w");
		printf("There are %d solution data arrays in this TIF file,listed below\n",solhead.sol_num);	
		for(int i=0;i<solhead.sol_num;i++)
			printf("\\%s  ",solhead.sol_name_array[i].c_str());
		
		printf("\n\nData will be written to %s as : x y soldata1 soldata2...",argv[1]);
		
		for(int i=0;i<node_array.size();i++)
		{ 
	  		fprintf(fp,"%e\t%e\t",node_array[soldata[i].index].x,node_array[soldata[i].index].y);
	  		for(int j=0;j<solhead.sol_num;j++)
				fprintf(fp,"%e\t",soldata[i].data_array[j]);
	  		fprintf(fp,"\n");
		}  
		fclose(fp);
	
		printf("finished\n");	
	}
	Tcl_ResetResult(interp);
	return TCL_OK;
}	

int C_save_CGNS(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{	
	if(argc!=2) return TCL_ERROR;
	if(region_array.size()>0)
	{
		printf("TIF dump to %s\n",argv[1]);	
		to_cgns(argv[1]);
	}
	Tcl_ResetResult(interp);
	return TCL_OK;
}

int C_save_parameter(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{		
	if(argc!=2) return TCL_ERROR;
	if(parameter_array.size()>0)
	{
		FILE *fp=fopen(argv[1],"w");
		int region_index=-1;
		for(int i=0;i<parameter_array.size();i++)
	 	{
	 		
	 		if(region_index != parameter_array[i].region-1)
	 		{
	 			region_index = parameter_array[i].region-1;
	 			fprintf(fp,"# %s \n",region_array[region_index].type);
	 		}	
	 	  	for(int j=0;j<parameter_array[i].parameter_name_array.size();j++)
		  		fprintf(fp,"%s = %e\n",parameter_array[i].parameter_name_array[j].c_str(),
			            		     parameter_array[i].parameter_value_array[j]);
		}
		fclose(fp);	            		     
        }
        printf("Parameters are written to %s\n",argv[1]);
        Tcl_ResetResult(interp);
	return TCL_OK;			            
}

int C_draw_data(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{
	OGL *ogl = (OGL*)pointer;
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	int mode = 2;
	if(argc==2) 
		mode = atoi(argv[1]); 
	if(argc==3) 
	{
		if(!strcmp(argv[2],"next")&&ogl->index_data<solhead.sol_num-1) ogl->index_data++;
		if(!strcmp(argv[2],"previous")&&ogl->index_data>0) ogl->index_data--;
	}				
	ogl->draw_data(ogl->index_data,mode);
	ogl->draw_axis();
	Tcl_ResetResult(interp);
	return TCL_OK;	
}

int C_draw_mesh(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{
	OGL *ogl = (OGL*)pointer;
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	if(tri_array.size())
		ogl->draw_mesh();
	else
		ogl->draw_logo();
	Tcl_ResetResult(interp);
	return TCL_OK;	
}

int C_save_screen(ClientData client_data, Tcl_Interp * interp, int argc, const char **argv)
{
	if(argc!=2) return TCL_ERROR;
	
	GLint viewport[4];
	TIFF *fout= TIFFOpen(argv[1], "w"); 
	int sampleperpixel = 4;    //RGB and alpha channel
	glGetIntegerv(GL_VIEWPORT,viewport);
	int width = viewport[2];
	int height = viewport[3];
	char *image=new char [width*height*sampleperpixel];
	glReadPixels(0,0,width,height,GL_RGBA,GL_UNSIGNED_BYTE,image);
	
	TIFFSetField(fout, TIFFTAG_IMAGEWIDTH, width);  // set the width of the image
	TIFFSetField(fout, TIFFTAG_IMAGELENGTH, height);    // set the height of the image
	TIFFSetField(fout, TIFFTAG_SAMPLESPERPIXEL, sampleperpixel);   // set number of channels per pixel
	TIFFSetField(fout, TIFFTAG_BITSPERSAMPLE, 8);    // set the size of the channels
	TIFFSetField(fout, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);    // set the origin of the image.
	TIFFSetField(fout, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(fout, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB); 
	
	tsize_t linebytes = sampleperpixel*width;     // length in memory of one row of pixel in the image. 
	// We set the strip size of the file to be size of one row of pixels
	TIFFSetField(fout, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(fout, width*sampleperpixel));
	//Now writing image to the file one strip at a time
	for (int row = 0; row < height; row++)
	{
    	        // check the index here, and figure out why not using h*linebytes
    		if(TIFFWriteScanline(fout, &image[(height-row-1)*linebytes], row, 0) < 0)
    		break;
	}
	TIFFClose(fout); 
	delete image;

	Tcl_ResetResult(interp);
	return TCL_OK;
}

int C_reset(ClientData pointer, Tcl_Interp * interp, int argc, const char **argv)
{
	OGL *ogl = (OGL*)pointer;
	ogl->reset();
	node_array.clear();
	edge_array.clear();
	tri_array.clear();
	region_array.clear();
	interface_array.clear();
	parameter_array.clear();
	solhead.sol_num = 0;
	solhead.sol_name_array.clear();
	soldata.clear();
	transol.data_name_array.clear();
	transol.data_unit_array.clear();
	transol.data_value_array.clear();
	Tcl_ResetResult(interp);
	return TCL_OK;	
}	
/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in the interp's result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AppInit(Tcl_Interp *interp)
{
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
    OGL *ogl = new OGL;   
    ogl->display = Tk_Display(Tk_MainWindow(interp));	
    ogl->interp = interp;
      
    if (Tkogl_Init(interp) == TCL_ERROR)
        return TCL_ERROR;

    Tcl_CreateCommand(interp, "C_read_TIF",C_read_TIF, ogl, NULL);	
    Tcl_CreateCommand(interp, "C_save_solution",C_save_solution, ogl, NULL);	
    Tcl_CreateCommand(interp, "C_save_CGNS",C_save_CGNS, ogl, NULL);
    Tcl_CreateCommand(interp, "C_save_parameter",C_save_parameter, ogl, NULL);
    Tcl_CreateCommand(interp, "C_draw_mesh",C_draw_mesh, ogl, NULL);
    Tcl_CreateCommand(interp, "C_draw_data",C_draw_data, ogl, NULL);
    Tcl_CreateCommand(interp, "C_save_screen",C_save_screen, ogl, NULL);
    Tcl_CreateCommand(interp, "C_reset",C_reset, ogl, NULL);
    return TCL_OK;
}

