#include "oglplot.h"
#include <GL/gl.h>
#include <GL/glu.h>

#ifdef LINUX
#include <GL/glx.h>
#endif
#include <math.h>
#include <string.h>
#include "tifdata.h"
#include "matdefine.h"

OGL::OGL()
{
	xlocation = 0;
	ylocation = 0;
	index_data = -1;
	char font_path[64];
	char font_name[] = "cour.ttf";
	sprintf(font_path,"%s/%s",getenv("CG_LIB_DIR"),font_name);
	monochrome10 = new Monochrome(font_path, 10);  
	monochrome12 = new Monochrome(font_path, 12);  
	monochrome14 = new Monochrome(font_path, 14);  
	monochrome36 = new Monochrome(font_path, 36);
	monochrome10->setForegroundColor(1.0, 1.0, 1.0);
	monochrome12->setForegroundColor(1.0, 1.0, 1.0);
	monochrome14->setForegroundColor(1.0, 1.0, 1.0);
	monochrome36->setForegroundColor(1.0, 1.0, 1.0);
	z = 0;
}

void OGL::reset()
{
	index_data = -1;
	delete [] z;
	z = 0;	
}		


    	
void OGL::set_post_bbox()
{
	xmin =  1e+38;
	xmax = -1e+38;
  	ymin =  1e+38;
  	ymax = -1e+38;
	//find bbox
  	for(int i=0;i<node_array.size();i++)
  	{
  		if(xmin>node_array[i].x) xmin = node_array[i].x;
  		if(xmax<node_array[i].x) xmax = node_array[i].x;
  		if(ymin>node_array[i].y) ymin = node_array[i].y;
  		if(ymax<node_array[i].y) ymax = node_array[i].y;
  	}
  	//adjust point location
  	double xc = (xmax+xmin)/2.0;
	double yc = (ymax+ymin)/2.0;
	for(int i=0;i<node_array.size();i++)
  	{
  		node_array[i].x -= xc;
  		node_array[i].y -= yc;
  	}
  	for(int i=0;i<component_array.size();i++)
  	{
  		component_array[i].xmin -= xc*1e-4;
  		component_array[i].xmax -= xc*1e-4;
  		component_array[i].ymin -= yc*1e-4;
  		component_array[i].ymax -= yc*1e-4;
  	}
  	
	xmin -=xc; 
	xmax -=xc;
	ymin -=yc; 
	ymax -=yc;
	// get box 
	double Xlength = xmax-xmin;
  	double Ylength = ymax-ymin;
	if (Xlength>Ylength)
  		box_length = 1.1*Xlength;
 	else 
		box_length = 1.1*Ylength;
	//set tcl 
  	if(xmin>xmax||ymin>ymax)
  		sprintf(Tcl_cmd_buf,"tcl_set_post_oglregion -1.0 1.0 -1.0 1.0");
  	else
  		sprintf(Tcl_cmd_buf,"tcl_set_post_oglregion %f %f %f %f",xmin,xmax,ymin,ymax);
  	
	if(Tcl_Eval(interp,Tcl_cmd_buf)==TCL_ERROR)
		printf("TCL tcl_set_post_oglregion error\n");
	
	// alloc z array
	z = new GLfloat[node_array.size()];	
		
}  

	
void OGL::print(char *s, int size, GLfloat x, GLfloat y, GLfloat z)
{
 	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	if(size==10)   monochrome10->draw(x, y, z, s);
	else if(size==12)   monochrome12->draw(x, y, z, s);
	else if(size==14)   monochrome14->draw(x, y, z, s);
	else if(size==36)   monochrome36->draw(x, y, z, s);
	else monochrome12->draw(x, y, z, s);	
}


void OGL::draw_logo()
{	
       print("TIF Tool v 1.4",36, -1, 0, 0);
}   

void OGL::draw_mesh()
{
 	double *x = new double[node_array.size()];
  	double *y = new double[node_array.size()];
	int  region_num = region_array.size();	 //total region;
  	int **local_index = new int*[region_num];    //reg_index = local_index[reg][global_index] 
  	int *conn_nodes = new int[region_num];         //for zone to zone connectivity node num
  	int **conn = new int*[region_num];           //for zone to zone connectivity information
 
  	for(int i=0; i<region_num; i ++)
  	{	 
  		local_index[i] = new int[node_array.size()];
  		conn[i] = new int[node_array.size()];
  	}	
 
  	int *af = new int[node_array.size()];
  
  	for(int i=0;i<region_num;i++)
  	{ 
  		for(int j = 0; j < node_array.size(); j++) local_index[i][j] = -1; //-1 means node not in this region
  		int *pf = af;
        	for(int j = 0; j < node_array.size(); j++) *pf++ = 0;        //visited flag array
        	region_array[i].node_num = 0;
        	region_array[i].tri_num = 0;
        	conn_nodes[i] = 0;
    
		for(int j = 0; j < tri_array.size(); j++)     //search for all the nodes
	  	  if (tri_array[j].region == i)
	  	{
	  		region_array[i].tri_num++;
	  		int nodeA = tri_array[j].c1;
	  		int nodeB = tri_array[j].c2;
	  		int nodeC = tri_array[j].c3;
	  		if (!af[nodeA]) { af[nodeA] = 1;local_index[i][nodeA] = region_array[i].node_num++; }
	  		if (!af[nodeB]) { af[nodeB] = 1;local_index[i][nodeB] = region_array[i].node_num++; }
	  		if (!af[nodeC]) { af[nodeC] = 1;local_index[i][nodeC] = region_array[i].node_num++; }
	  	}
  	}
  	delete [] af;	
  	
	for(int i=0;i<region_num;i++)
  	{
 	
  		// get grid coordinates 
       		for(int j=0;j<node_array.size();j++)
    	  	  if(local_index[i][j]>-1)  //process node in this region 
  		{	
  		  
  		  x[local_index[i][j]] = node_array[j].x;
        	  y[local_index[i][j]] = node_array[j].y;
       		}
        	// draw each region
		if(IsInsulator(region_array[i].type))
	        	glColor3f(0.0,1.0,1.0);
	        else if(IsElectrode(region_array[i].type))
	        	glColor3f(0.0,0.0,1.0);
	        else
	        	glColor3f(0.3,0.7,0.0);		
	        
  		for(int j = 0; j < tri_array.size(); j++)
     	 	  if (tri_array[j].region == i)  //is the triangle belongs to this region?
   		{
	    	  int p1 =  local_index[i][tri_array[j].c1];
	    	  int p2 =  local_index[i][tri_array[j].c2];
	    	  int p3 =  local_index[i][tri_array[j].c3];
	          glBegin(GL_LINE_LOOP);
	      	  glVertex3f(x[p1],y[p1],0.0);
	      	  glVertex3f(x[p2],y[p2],0.0);
	      	  glVertex3f(x[p3],y[p3],0.0);
	      	  glEnd();  
		}
	}
	
	delete [] x;
  	delete [] y;
	for(int i =0; i<region_num; i ++) 
  	{
     		delete [] local_index[i];
  		delete [] conn[i];
  	}
  	delete  [] local_index;	 
  	delete  [] conn;
  	delete  [] conn_nodes;
  	
	sprintf(Tcl_cmd_buf,"tcl_set_solname %s","mesh");
  	if(Tcl_Eval(interp,Tcl_cmd_buf)==TCL_ERROR)
		printf("TCL tcl_set_solname error\n");
}

void OGL::draw_axis()
{
	//draw axis
	glColor3f(1.0,1.0,1.0);
	glBegin (GL_LINE_LOOP);
		glVertex3f(xmin,ymin,0);
		glVertex3f(xmax,ymin,0);
		glVertex3f(xmax,ymax,0);
		glVertex3f(xmin,ymax,0);
	glEnd();
	glBegin (GL_LINES);
		glVertex3f(xmin,ymin,0);
		glVertex3f(xmin,ymin,0.4*box_length);
		
		glVertex3f(xmax,ymin,0);
		glVertex3f(xmax,ymin,0.4*box_length);
		
		glVertex3f(xmax,ymax,0);
		glVertex3f(xmax,ymax,0.4*box_length);
		
		glVertex3f(xmin,ymax,0);
		glVertex3f(xmin,ymax,0.4*box_length);
	glEnd();
	
	glBegin (GL_LINE_LOOP);
		glVertex3f(xmin,ymin,0.4*box_length);
		glVertex3f(xmax,ymin,0.4*box_length);
		glVertex3f(xmax,ymax,0.4*box_length);
		glVertex3f(xmin,ymax,0.4*box_length);
	glEnd();
	
	GLfloat d = 0.01*box_length;
	glBegin (GL_LINES);
		glVertex3f(xmin,ymin,0);
		glVertex3f(xmin,ymin-d,0);
		
		glVertex3f(xmin+0.25*(xmax-xmin),ymin,0);
		glVertex3f(xmin+0.25*(xmax-xmin),ymin-d,0);
		
		glVertex3f(xmin+0.5*(xmax-xmin),ymin,0);
		glVertex3f(xmin+0.5*(xmax-xmin),ymin-d,0);
		
		glVertex3f(xmin+0.75*(xmax-xmin),ymin,0);
		glVertex3f(xmin+0.75*(xmax-xmin),ymin-d,0);
		
		glVertex3f(xmax,ymin,0);
		glVertex3f(xmax,ymin-d,0);
		
		glVertex3f(xmin,ymin,0);
		glVertex3f(xmin-d,ymin,0);
		
		glVertex3f(xmin,ymin+0.25*(ymax-ymin),0);
		glVertex3f(xmin-d,ymin+0.25*(ymax-ymin),0);
		
		glVertex3f(xmin,ymin+0.5*(ymax-ymin),0);
		glVertex3f(xmin-d,ymin+0.5*(ymax-ymin),0);
		
		glVertex3f(xmin,ymin+0.75*(ymax-ymin),0);
		glVertex3f(xmin-d,ymin+0.75*(ymax-ymin),0);
		
		glVertex3f(xmin,ymax,0);
		glVertex3f(xmin-d,ymax,0);
	glEnd();
	
	//draw lable
	char buf[32];
	
	sprintf(buf,"%2.1f",xmin);
	print(buf, 10, xmin, ymin-0.03*box_length,0);
	sprintf(buf,"%2.1f",xmin+0.25*(xmax-xmin));
	print(buf, 10, xmin+0.25*(xmax-xmin), ymin-0.03*box_length,0);
	sprintf(buf,"%2.1f",xmin+0.5*(xmax-xmin));
	print(buf, 10, xmin+0.5*(xmax-xmin), ymin-0.03*box_length,0);
	sprintf(buf,"%2.1f",xmin+0.75*(xmax-xmin));
	print(buf, 10, xmin+0.75*(xmax-xmin), ymin-0.03*box_length,0);
	sprintf(buf,"%2.1f",xmax);
	print(buf, 10, xmax, ymin-0.04*box_length,0);
	print("X", 14, (xmin+xmax)/2, ymin-0.08*box_length,0);

	sprintf(buf,"%2.1f",ymin);
	print(buf, 10, xmin-0.08*box_length, ymin,0);
	sprintf(buf,"%2.1f",ymin+0.25*(ymax-ymin));
	print(buf, 10, xmin-0.08*box_length, ymin+0.25*(ymax-ymin),0);
	sprintf(buf,"%2.1f",ymin+0.5*(ymax-ymin));
	print(buf, 10, xmin-0.08*box_length, ymin+0.5*(ymax-ymin),0);
	sprintf(buf,"%2.1f",ymin+0.75*(ymax-ymin));
	print(buf, 10, xmin-0.08*box_length, ymin+0.75*(ymax-ymin),0);
	sprintf(buf,"%2.1f",ymax);
	print(buf, 10, xmin-0.08*box_length, ymax,0);
	print("Y", 14, xmin-0.12*box_length, (ymin+ymax)/2,0);
	
	sprintf(buf,"%5.4e",vmax);
	print(buf, 10, xmax+0.01*box_length, ymin-0.01*box_length, 0.4*box_length);
	sprintf(buf,"%5.4e",(vmax+vmin)/2);
	print(buf, 10, xmax+0.01*box_length, ymin-0.01*box_length, 0.2*box_length);
	sprintf(buf,"%5.4e",vmin);
	print(buf, 10, xmax+0.01*box_length, ymin-0.01*box_length, 0);
	
	print("Z", 14, xmax, ymin, 0.42*box_length);
}

	
void OGL::draw_data(int d,int mode)
{
	vmax = -1e+100;
	vmin =  1e+100;
	GLfloat color[3];
	
	if(!soldata.size()) return;
	
	sprintf(Tcl_cmd_buf,"tcl_set_solname %s",solhead.sol_name_array[d].c_str());
  	if(Tcl_Eval(interp,Tcl_cmd_buf)==TCL_ERROR)
		printf("TCL tcl_set_solname error\n");
			
	for(int i=0;i<soldata.size();i++)
	{ 
		if(vmax<soldata[i].data_array[d]) vmax = soldata[i].data_array[d];
		if(vmin>soldata[i].data_array[d]) vmin = soldata[i].data_array[d];	
	}  
	if(vmax!=vmin)
	  for(int i=0;i<soldata.size();i++)
	  {
		int node_index = soldata[i].index;
		z[node_index] = 0.4*box_length*log10(1+(soldata[i].data_array[d]-vmin)/(vmax-vmin))/log10(2);
	  }
	else  
	  for(int i=0;i<soldata.size();i++)
	  {
		int node_index = soldata[i].index;
		z[node_index] = 0.0;
	  }
	// draw 3D data
	for(int i=0;i<tri_array.size();i++)
	{
	    int p1 =  tri_array[i].c1;
	    int p2 =  tri_array[i].c2;
	    int p3 =  tri_array[i].c3;
	    glBegin (GL_TRIANGLES);
	      rainbow.getcolor3fv(mode,z[p1],color);
	      glColor3fv(color);
	      glVertex3f(node_array[p1].x,node_array[p1].y,z[p1]);
	      rainbow.getcolor3fv(mode,z[p2],color);
	      glColor3fv(color);
	      glVertex3f(node_array[p2].x,node_array[p2].y,z[p2]);
	      rainbow.getcolor3fv(mode,z[p3],color);
	      glColor3fv(color);
	      glVertex3f(node_array[p3].x,node_array[p3].y,z[p3]);
	    glEnd();  
	}
}
     

