/*****************************************************************************/
/*   	        8888888         88888888         88888888                    */
/*  	      8                8                8                            */
/* 	     8                 8                8                            */
/*  	     8                  88888888         88888888                    */
/* 	     8      8888                8                8                   */
/* 	      8       8                 8                8                   */
/* 	        888888         888888888        888888888                    */
/*                                                                           */
/*       A Two-Dimensional General Purpose Semiconductor Simulator.          */
/*                                                                           */
/*  GSS 0.4x                                                                 */
/*  Last update: Oct 17, 2005                                                */
/*                                                                           */
/*  Gong Ding                                                                */
/*  gdiso@ustc.edu                                                           */
/*  NINT, No.69 P.O.Box, Xi'an City, China                                   */
/*                                                                           */
/*****************************************************************************/

#include "rainbow.h"
#include <math.h>
 
static int 
color20[20][3]=	 {{255,   0,   0},
		 {255, 103,   0},
		 {255, 148,   0},
		 {255, 178,   0},
		 {255, 207,   0},
		 {255, 233,   0},
		 {255, 255,   0},
		 {220, 255,   0},
		 {180, 255,   0},
		 {125, 255,   0},
		 {0,   255,   0},
		 {0,   255, 127},
		 {0,   255, 179},
		 {0,   255, 221},
		 {0,   255, 255},
		 {0,   228, 255},
		 {0,   197, 255},
		 {0,   161, 255},
		 {0,   114, 255},
		 {0,     0, 255}};	
 	
Rainbow::Rainbow()
{
	min = 0.0;
	max = 1.0;
	
}	
  
Rainbow::Rainbow(REAL min_value,REAL max_value)
{
	min = min_value;
	max = max_value;	
}	


void Rainbow::getcolor3fv(int mode, REAL value,GLfloat*color)
{
	if(mode==1) getcolor3fv1(value,color);
	if(mode==2) getcolor3fv2(value,color);
	if(mode==3) getcolor3fv3(value,color);
	if(mode==4) getcolor3fv4(value,color);
}	

  	
void Rainbow::getcolor3fv1(REAL value,GLfloat* color)
{
	int color_index = (int)((1.0-value)*20);
	if(color_index<0) color_index=0;
	if(color_index>19) color_index=19;
	color[0]=color20[color_index][0]/255.0;
	color[1]=color20[color_index][1]/255.0;
	color[2]=color20[color_index][2]/255.0;
}


void Rainbow::getcolor3fv2(REAL n, GLfloat* color )
{
	if( n < 0.5 * max)  //calculate the color by the intension at this point
	{
		color[0] = (n - min) / (max - min);
		color[1] = 2 * (n - min) / (max - 2 * min);
		color[2] = (2 * n -max) / (2 * min -max);		
	}
	else
	{
		color[0] = (n - min) / (max - min);
		color[1] = 2 * (max - n) / max;
		color[2] = 0; 
	}
}

void Rainbow::getcolor3fv3(REAL n, GLfloat* color)
{

	if( n < 0.5 )  //calculate the color by the intension at this point
	{
		color[0] = 0.5 * n + 0.5;
		color[1] = 2 * n;
		color[2] = 1 - 2 * n;		
	}
	else
	{
		color[0] = 0.5 * n + 0.5;
		color[1] = 2-2 * n;
		color[2] = 0;
	}

}


void Rainbow::getcolor3fv4( REAL n, GLfloat* color)
{
	REAL t = 0.5*(max + min);
	color[0] = (n - min) / (max - min);
	color[1] = 1 - exp( -80 * (n - min) / (max -min));
	color[2] = exp( -80 * (n - min) / (max -min));

	if( n > t)
		color[1] = (n - max) / (t - max);

}

