@echo off
setlocal

set copts=
set debug=
set do64bit=0
set cgnsdir=
set tcldir=
set tkdir=
set tclinc=
set tcllib=
set tklib=
set adfhdir=
set instdir=c:\progra~1\adfviewer

:next
if "%1" == "" goto doit
if %1 == -help goto usage

rem ----- compiler options

if %1 == -ML (
  echo using -ML : single-threaded library
  set copts=%1
  shift
  goto next
)
if %1 == -MD (
  echo using -MD : multi-threaded library with RTL
  set copts=%1
  shift
  goto next
)
if %1 == -MT (
  echo using -MT : multi-threaded library
  set copts=%1
  shift
  goto next
)
if %1 == -debug (
  echo debug is enabled
  set debug=-Zi
  shift
  goto next
)
if %1 == -64 (
  echo build 64-bit version
  set do64bit=1
  shift
  goto next
)

rem ----- cgns directory

if not %1 == -cgns goto adfh
shift
if "%1" == "" (
  echo ERROR:CGNS directory arg to -cgns not given
  goto usage
)
if not exist %1\cgnslib.h (
  echo ERROR:cgnslib.h not found in %1
  goto done
)
set cgnsdir=%~f1
echo using CGNS directory %cgnsdir%
shift
goto next

rem ----- adfh directory

:adfh
if not %1 == -adfh goto tcl
shift
if "%1" == "" (
  echo ERROR:adfh directory arg to -adfh not given
  goto usage
)
if not exist %1\src\ADF.h (
  echo ERROR:can't find src\ADF.h in %1
  goto done
)
set adfhdir=%~f1
echo using adfh directory %adfhdir%
shift
goto next

rem ----- tcl directory

:tcl
if not %1 == -tcl goto tk
shift
if "%1" == "" (
  echo ERROR:tcl directory arg to -tcl not given
  goto usage
)
if not exist %1\generic\tcl.h (
  echo ERROR:can't find generic\tcl.h in %1
  goto done
)
set tcldir=%~f1
echo using tcl directory %tcldir%
shift
goto next

rem ----- tk directory

:tk
if not %1 == -tk goto install
shift
if "%1" == "" (
  echo ERROR:tk directory arg to -tk not given
  goto usage
)
if not exist %1\generic\tk.h (
  echo ERROR:can't find generic\tk.h in %1
  goto done
)
set tkdir=%~f1
echo using tk directory %tkdir%
shift
goto next

rem ----- installation directory

:install
if not %1 == -install goto doit
shift
if "%1" == "" (
  echo ERROR:installation directory arg to -install not given
  goto usage
)
set instdir=%~f1
echo installation to %instdir%
shift
goto next

rem ----- print usage

:badarg
echo ERROR:unknown argument %1
:usage
echo usage: configure [options]
echo options:
echo   -ML           : single-threaded using libc.lib (default)
echo   -MT           : multi-threaded using libcmt.lib
echo   -MD           : multi-threaded using mscvrt.lib and mscvrt.dll
echo   -debug        : add debugging to library
echo   -64           : build 64-bit version
echo   -cgns cgnsdir : specify the CGNS directory
echo   -adfh adfhdir : specify the adfh directory
echo   -tcl tcldir   : specify the Tcl source directory
echo   -tk tkdir     : specify the Tk source directory
echo   -install dir  : specify installation directory
echo                   (default c:\program files\adfviewer)
goto done

:doit

rem ----- CGNS setup

if not "%adfhdir%" == "" (
  set setup=adfh
  goto gettcltk
)
if not "%cgnsdir%" == "" goto getcgns

echo checking for CGNS ...
for /D %%d in ( %~d0\*.* ) do (
  if exist %%d\cgnslib.h (
    echo %%d
    set cgnsdir=%%d
    goto getcgns
  )
  for /D %%e in ( %%d\*.* ) do (
    if exist %%e\cgnslib.h (
      echo %%e
      set cgnsdir=%%e
      goto getcgns
    )
    for /D %%f in ( %%e\*.* ) do (
      if exist %%f\cgnslib.h (
        echo %%f
        set cgnsdir=%%f
        goto getcgns
      )
      for /D %%g in ( %%f\*.* ) do (
        if exist %%g\cgnslib.h (
          echo %%g
          set cgnsdir=%%g
          goto getcgns
        )
      )
    )
  )
)
echo ERROR:couldn't find CGNS directory
goto done

:getcgns
set setup=old
if exist %cgnsdir%\f2c\nul set setup=new

:gettcltk
if exist c:\progra~1\tcl\include\nul (
  set tclinc=-Ic:\progra~1\tcl\include
  for %%i in ( 81 82 83 84 85 86 ) do (
    if exist c:\progra~1\tcl\lib\tcl%%i.lib set tcllib=c:\progra~1\tcl\lib\tcl%%i.lib
    if exist c:\progra~1\tcl\lib\tk%%i.lib set tklib=c:\progra~1\tcl\lib\tk%%i.lib
  )
)

set tksource=yes
if "%tcldir%" == "" set tksource=no
if "%tkdir%" == "" set tksource=no
if %tksource% == no (
  echo Tcl and/or Tk source directory not specified.
  echo You will not be able to build cgnsplot without them.
  set tcldir=%~d0\tcl8.3.1
  set tkdir=%~d0\tk8.3.1
) else (
  set tclinc=-I%tcldir%\generic -I%tkdir%\generic -I%tkdir%\xlib
  for %%i in ( 81 82 83 84 85 86 ) do (
    if exist %tcldir%\win\Release\tcl%%i.lib set tcllib=%tcldir%\win\Release\tcl%%i.lib
    if exist %tkdir%\win\Release\tk%%i.lib set tklib=%tkdir%\win\Release\tk%%i.lib
  )
)

set doedit=no
if "%tclinc%" == "" (
  set tclinc=-Ic:\progra~1\tcl\include
  set tcllib=c:\progra~1\tcl\lib\tcl83.lib
  set tklib=c:\progra~1\tcl\lib\tk83.lib
  echo couldn't get Tcl/Tk includes or libraries
  set doedit=yes
)
if "%tcllib%" == "" (
  set tcllib=c:\progra~1\tcl\lib\tcl83.lib
  echo couldn't determine Tcl library
  set doedit=yes
)
if "%tklib%" == "" (
  set tklib=c:\progra~1\tcl\lib\tk83.lib
  echo couldn't determine Tk library
  set doedit=yes
)
if %doedit% == yes echo You need to edit make.win and set these appropiately

if %do64bit% == 1 (
  if "%copts%" == "" set copts=-MT
  if "%copts%" == "-ML" set copts=-MT
  set clibs=bufferoverflowu.lib
  set windir=WIN64
) else (
  if "%copts%" == "" set copts=-ML
  set clibs=
  set windir=WIN32
)

if not "%debug%" == "" (
  if "%copts%" == "" (
    set copts=%debug%
  ) else (
    set copts=%copts%d %debug%
  )
  set lopts=/debug
) else (
  set lopts=/release
)

rem ----- create Makefile

echo creating Makefile
echo # makefile for the CGNS tools under Windows> Makefile
echo.>> Makefile
echo MAKE = nmake -nologo -f Makefile.win>> Makefile
echo.>> Makefile
if %tksource% == yes (
  echo default : all>> Makefile
  echo install : install-all>> Makefile
) else (
  echo default : adfviewer cgnscalc tools utilities>> Makefile
  echo install : install-adfviewer install-cgnscalc \>> Makefile
  echo 	install-tools install-utilities>> Makefile
)
echo.>> Makefile
echo all : adfviewer cgnscalc cgnsplot tools utilities>> Makefile
echo install-all : install-adfviewer install-cgnscalc install-cgnsplot \>> Makefile
echo 	install-tools install-utilities>> Makefile
echo.>> Makefile
echo clean :>> Makefile
echo 	cd adfviewer ^&^& $(MAKE) clean>> Makefile
echo 	cd cgnscalc ^&^& $(MAKE) clean>> Makefile
echo 	cd cgnsplot ^&^& $(MAKE) clean>> Makefile
echo 	cd tools ^&^& $(MAKE) clean>> Makefile
echo 	cd utilities ^&^& $(MAKE) clean>> Makefile
echo 	cd calclib ^&^& $(MAKE) clean>> Makefile
echo 	cd tkogl ^&^& $(MAKE) clean>> Makefile
echo.>> Makefile
echo adfviewer : prog-adfviewer>> Makefile
echo cgnscalc  : prog-cgnscalc>> Makefile
echo cgnsplot  : prog-cgnsplot>> Makefile
echo tools	  : prog-tools>> Makefile
echo utilities : prog-utilities>> Makefile
echo.>> Makefile
echo prog-adfviewer :>> Makefile
echo 	cd adfviewer ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo prog-cgnscalc : lib-calclib>> Makefile
echo 	cd cgnscalc ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo prog-cgnsplot : lib-tkogl>> Makefile
echo 	cd cgnsplot ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo prog-tools :>> Makefile
echo 	cd tools ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo prog-utilities : lib-calclib>> Makefile
echo 	cd utilities ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo lib-calclib :>> Makefile
echo 	cd calclib ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo lib-tkogl :>> Makefile
echo 	cd tkogl ^&^& $(MAKE)>> Makefile
echo.>> Makefile
echo install-adfviewer :>> Makefile
echo 	cd adfviewer ^&^& $(MAKE) install>> Makefile
echo.>> Makefile
echo install-cgnscalc : lib-calclib>> Makefile
echo 	cd cgnscalc ^&^& $(MAKE) install>> Makefile
echo.>> Makefile
echo install-cgnsplot : lib-tkogl>> Makefile
echo 	cd cgnsplot ^&^& $(MAKE) install>> Makefile
echo.>> Makefile
echo install-tools :>> Makefile
echo 	cd tools ^&^& $(MAKE) install>> Makefile
echo.>> Makefile
echo install-utilities : lib-calclib>> Makefile
echo 	cd utilities ^&^& $(MAKE) install>> Makefile

rem ----- create make.win

echo creating make.win
echo # makefile include for %windir%> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # CGNS and ADF/HDF5 setup>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
if %setup% == adfh (
  echo ADFHDIR  = %adfhdir%>> make.win
  echo !include $^(ADFHDIR^)\make.%windir%>> make.win
  echo ADFINC   = $^(ADFHDIR^)\src>> make.win
  echo CGNSLIB  = $^(ADFHDIR^)\%windir%\$^(LIBCGNS^)>> make.win
  echo HDF5LIBS = $^(HDF5LIB^) $^(SZIPLIB^) $^(ZLIBLIB^)>> make.win
)
if %setup% == old (
  echo CGNSDIR  = %cgnsdir%>> make.win
  echo !include $^(CGNSDIR^)\make.%windir%>> make.win
  echo ADFINC   = $^(CGNSDIR^)\adf>> make.win
  echo CGNSLIB  = $^(CGNSDIR^)\lib\$^(LIBCGNS^)>> make.win
  echo HDF5LIBS =>> make.win
)
if %setup% == new (
  echo CGNSDIR  = %cgnsdir%>> make.win
  echo !include $^(CGNSDIR^)\make.%windir%>> make.win
  echo ADFINC   = $^(CGNSDIR^)\$^(ADFDIR^)>> make.win
  echo CGNSLIB  = $^(CGNSDIR^)\$^(LIBCGNS^)>> make.win
  echo HDF5LIBS = $^(HDF5LIB^) $^(SZIPLIB^) $^(ZLIBLIB^)>> make.win
)
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # where and how to install>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo INSTALLDIR = %instdir%>> make.win
echo INSTALL    = copy /b>> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # path to the standard Tcl/Tk includes and libraries>> make.win
echo # this may either be the source distribution>> make.win
echo # or the installed version>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo TCLINC = %tclinc%>> make.win
echo TCLLIB = %tcllib%>> make.win
echo TKLIB  = %tklib%>> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # to build TkOGL, need includes from source distribution>> make.win
echo # TKOGLINCS give the include directories>> make.win
echo # TKOGLLIB is the library relative to cgnsplot directory>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo TCLSOURCE = %tcldir%>> make.win
echo TKSOURCE  = %tkdir%>> make.win
echo TKOGLINCS = -I$(TKSOURCE)\win -I$(TKSOURCE)\generic -I$(TKSOURCE)\xlib \>> make.win
echo             -I$(TCLSOURCE)\win -I$(TCLSOURCE)\generic>> make.win
echo.>> make.win
echo TKOGLLIB = ..\tkogl\tkogl.lib>> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # set to trap math errors in calculator>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo MATHERR = -DUSE_MATHERR>> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # compiler and linker options>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo CC     = cl>>make.win
echo COPTS  = /nologo -DPROTOTYPE %copts%>>make.win
echo LINK   = link>> make.win
echo LFLAGS = /nologo %lopts%>> make.win
echo.>> make.win
echo #------------------------------------------------------->> make.win
echo # windows libraries>> make.win
echo #------------------------------------------------------->> make.win
echo.>> make.win
echo dlllibs = gdi32.lib comdlg32.lib>> make.win
echo.>> make.win
echo guilibs	= libc.lib oldnames.lib kernel32.lib advapi32.lib \>> make.win
echo 	user32.lib gdi32.lib comdlg32.lib winspool.lib>> make.win
echo.>> make.win
echo ogllibs = opengl32.lib glu32.lib>> make.win

:done
endlocal

